package MT::SheetAsset::Model::Worksheet;

use strict;
use warnings;
use base qw( MT::ErrorHandler );

use MT::SheetAsset::Util;

sub new {
    my $package = shift;
    my $opts = shift || {};
    my $self = $package->SUPER::new(@_);

    $self->{name} = $opts->{name};
    $self->{rows} = $self->normalize_rows($opts->{rows} || []);

    $self->{name} = '' unless defined $self->{name};

    $self;
}

sub normalize_rows {
    my $self = shift;
    my ( $rows ) = @_;

    return [] if ref $rows ne 'ARRAY';
    my @normalized = map {
        my $row = $_;
        if ( ref $row eq 'ARRAY' ) {
            $row = [ map {
                defined($_) ? $_ : '';
            } @$row ];
        } else {
            $row = [];
        }
        $row;
    } @$rows;

    \@normalized;
}

sub to_hash {
    my $self = shift;

    my %hash = (
        name => $self->{name} || '',
        rows => $self->{rows} || [],
    );

    \%hash;
}

sub name {
    my $self = shift;
    $self->{name} = $_[0] if @_;
    $self->{name};
}

sub workbook {
    my $self = shift;
    $self->{workbook} = $_[0] if @_;
    $self->{workbook};
}

sub rows {
    my $self = shift;
    $self->{rows} = $self->normalize_rows($_[0]) if @_;
    $self->{rows};
}

sub scalar_grid {
    my $self = shift;
    my $opts = shift || {};

    $self->{rows};
}

sub hashes {
    my $self = shift;
    my $opts = shift || {};

    my $skip_headers = $opts->{skip_headers} || 0;
    my $skip_rows = $opts->{skip_rows} || 0;

    my $grid = $self->scalar_grid;

    my $headers = $grid->[$skip_headers] || [];
    my $rows_count = scalar @$grid;
    my $cols_count = scalar @$headers;
    my @hashes;
    for ( my $i = $skip_headers + $skip_rows + 1; $i < $rows_count; $i++ ) {
        my $row = $grid->[$i] || last;
        my $hash = {};
        @$hash{@$headers} = @$row;
        push @hashes, $hash;
    }

    \@hashes;
}

sub window {
    my $self = shift;
    my $opts = shift || {};

    if ( my $rows = $opts->{rows} ) {
        splice @{$self->{rows}}, $rows;
    }

    if ( my $cols = $opts->{columns} || $opts->{cols} ) {
        foreach my $row ( @{$self->{rows}} ) {
            splice @$row, $cols;
        }
    }

    $self;
}

sub table_html {
    my $self = shift;
    my $opts = shift || {};

    my $table_class = $opts->{table_class} || '';
    my $row_class = $opts->{row_class} || '';
    my $column_class = $opts->{column_class} || '';

    my $html;
    $html = qq{<table class="$table_class">};
    $html .= q{<tbody>};

    my $r = 0;
    foreach my $row ( @{$self->rows} ) {
        my $even_odd = ($r % 2) ? 'even' : 'odd';
        $r++;

        $html .= qq{<tr class="$row_class $even_odd sheetasset-row-$r">};
        my $c = 0;
        foreach my $col ( @$row ) {
            $c++;
            $col = '' unless defined $col;
            $html .= qq{<td class="$column_class sheetasset-col-$c">} . $col . '</td>';
        }
        $html .= '</tr>';
    }
    $html .= q{</tbody></table>};

    $html;
}

sub address {
    my $self_class = shift;
    my $address = shift;
    $address =~ s/^(\s+)|(\s+)$//g;
    my ( $c, $r ) = ( $address =~ /^([a-z]*)([0-9]*)$/i );

    $r ||= 0;

    if ( $c ) {
        my $base = ord('a') - 1;
        my @cols = reverse split(//, lc($c));
        my $value = 0;
        my $place = 1;
        foreach my $digit ( @cols ) {
            $value += $place * ( ord($digit) - $base );
            $place *= 26;
        }
        $c = $value;
    } else {
        $c = 0;
    }

    ( $r, $c );
}

sub point_cell {
    my $self = shift;
    my $args = shift || {};

    if ( my $address = $args->{address} || $args->{cell} ) {
        ( $args->{row}, $args->{col} ) = $self->address($address);
    }

    $self->lookup_cell($args->{row}, $args->{col} || $args->{column});
}

sub lookup_cell {
    my $self = shift;
    my ( $r, $c ) = @_;

    return undef if !$r || !$c;

    my $row = $self->{rows}->[$r - 1] || return undef;
    my $col = $row->[$c - 1] || return undef;

    $col;
}

1;