package MT::SheetAsset::Tags;

use strict;
use warnings;
use MT::SheetAsset::Util;
use MT::SheetAsset::Type::Base;
use MT::SheetAsset::URLHandler::Base;

sub require_current_workbook {
    my ( $ctx, $args, $cond ) = @_;

    return $ctx->{__stash}{sheetasset_workbook}
        if defined $ctx->{__stash}{sheetasset_workbook};

    my $cache = MT->instance->request->cache || {};
    my $wb;

    # tag to something
    if ( defined( my $tag = $args->{tag} ) ) {
        $tag =~ s/^MT:?//i;
        require Storable;
        my $local_args = Storable::dclone($args);
        my $value = $ctx->tag( $tag, $local_args, $cond );

        if ( $value =~ m!^https?://! ) {
            delete $args->{tag};
            $args->{url} = $value;
        } else {
            # Parse as JSON
            my $json;
            eval { $json = MT::Util::from_json($value); };
            if ( $json && ref $json eq 'HASH' ) {
                if ( my $data = $json->{data} ) {
                    $wb = MT::SheetAsset::Model::Workbook->new;
                    my $ws = MT::SheetAsset::Model::Worksheet->new({ name => '', rows => $data });
                    $wb->add_worksheet($ws);
                    return $wb;
                }
            }
        }
    }

    # URL to File
    my $file;
    if ( defined( my $url = $args->{url} ) ) {
        my $cache = $cache->{sheetasset_workbook_url_cache} ||= {};
        return $cache->{$url} if $cache->{$url};

        my %ctx = ( url => $url );
        my $handler = url_handler( \%ctx );
        if ( $file = $handler->download( \%ctx ) ) {
            # Pass to file attribute context
        } else {
            my $handler = $ctx{url_handler} or return $ctx->error(pluign->translate('No URL Handler')); # TODO translate
            return $ctx->error($handler->errstr);
        }
    }

    if ( defined( $file ||= $args->{file} ) ) {
        my $cache = $cache->{sheetasset_workbook_file_cache} ||= {};
        return $cache->{$file} if $cache->{$file};

        return $ctx->error(plugin->translate('File [_1] is not readable.', $file))
            if !-e $file || !-r $file;

        my $asset = MT::SheetAsset::Type::Base->sheetasset_from_file($file)
            or $ctx->error(plugin->translate('File [_1] is not a spreadsheet.', $file));
        $wb = $asset->sheetasset_workbook($args)
            or return $ctx->error($asset->error);
        $cache->{$file} = $wb;
    } elsif ( defined( my $asset = $ctx->stash('asset') ) ) {
        my $cache = $cache->{sheetasset_workbook_asset_cache} ||= {};
        return $cache->{$asset->id} if $asset->id && $cache->{$asset->id};

        if ( $asset->can('sheetasset_workbook') ) {
            $wb = $asset->sheetasset_workbook($args)
                or return $ctx->error($asset->errstr);
            $cache->{$asset->id} = $wb if $asset->id;
        }
    }

    $wb or $ctx->error(plugin->translate('No SheetAsset workbook context.'));
}

sub require_current_worksheet {
    my ( $ctx, $args ) = @_;

    return $ctx->{__stash}{sheetasset_worksheet}
        if defined $ctx->{__stash}{sheetasset_worksheet};

    my $wb = require_current_workbook(@_) or return;
    my $alt_attr = $args->{_alt_attr_} || 'worksheet';

    if ( defined ( my $name = $args->{$alt_attr} || $args->{worksheet} ) ) {
        return $wb->find_worksheet($name)
            || $ctx->error(plugin->translate('No worksheet named "[_1]" in workbook "[_2]".', $name, $wb->filepath));
    }

    if ( defined ( my $index = $args->{worksheet_index} ) ) {
        return $wb->worksheets->[$index - 1]
            || $ctx->error(plugin->translate('No worksheet indexed [_1] in workbook "[_2]".', $index, $wb->filepath));
    }

    my $ws = $wb->current_worksheet()
        || return $ctx->error(plugin->translate('No current worksheet in "[_1]".', $wb->filepath));

    $ws;
}

sub require_current_row {
    my ( $ctx, $args ) = @_;
    $ctx->{__stash}{sheetasset_row} || $ctx->error(plugin->translate('No SheetAsset row context.'));
}

sub require_current_cell {
    my ( $ctx, $args ) = @_;

    return $ctx->{__stash}{sheetasset_cell}
        if defined $ctx->{__stash}{sheetasset_cell};

    my $ws = require_current_worksheet(@_) or return;
    defined( my $cell = $ws->point_cell($args) )
        or return $ctx->error('No cell context');

    $cell;
}

sub require_current_record {
    my ( $ctx, $args ) = @_;
    $ctx->{__stash}{sheetasset_record} || $ctx->error(plugin->translate('No SheetAsset record context.'));
}

sub hdlr_SheetAssetWorksheetName {
    my ( $ctx, $args ) = @_;

    my $ws = require_current_worksheet(@_) or return;

    $ws->name || '';
}

sub hdlr_IfSheetAssetAvailable {
    my ( $ctx, $args, $cond ) = @_;
    local $ctx->{_errstr};
    require_current_worksheet(@_) ? 1 : 0;
}

sub hdlr_SheetAssetWorksheet {
    my ( $ctx, $args ) = @_;

    $args->{_alt_attr_} = 'name';
    my $ws = require_current_worksheet(@_) or return;

    local $ctx->{__stash}{sheetasset_worksheet} = $ws;

    general_build(@_);
}

sub _window_array {
    my ( $ctx, $args, $array ) = @_;

    my $offset = $args->{offset};
    my $limit = $args->{limit};

    return $array unless grep { defined $_ } ( $offset, $limit );

    my @windowed = @$array;
    @windowed = splice(@windowed, $offset || 0) unless defined $limit;
    @windowed = splice(@windowed, $offset || 0, $limit) if defined $limit;

    \@windowed;
}

sub general_build {
    my ( $ctx, $args, $cond ) = @_;
    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');

    defined( my $result = $builder->build($ctx, $tokens, $cond) )
        || return $ctx->error($builder->errstr);

    $result;
}

sub general_loop {
    my ( $array, $stash_key, $ctx, $args, $cond ) = @_;

    $array = _window_array($ctx, $args, $array);

    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');
    my $result = '';

    my $size = scalar @$array;
    my $vars = $ctx->{__stash}{vars} ||= {};
    for( my $i = 0; $i < $size; $i++ ) {
        my $item = $array->[$i];

        local $ctx->{__stash}->{$stash_key} = $item;
        local $vars->{__value__} = $item;
        local $vars->{__first__} = ( $i == 0 )? 1: 0;
        local $vars->{__last__} = ( $i == $size - 1 )? 1: 0;
        local $vars->{__odd__} = ( $i % 2 ) == 0;
        local $vars->{__even__} = ( $i % 2 ) == 1;
        local $vars->{__counter__} = $i + 1;

        defined( my $partial = $builder->build($ctx, $tokens, $cond) )
            || return $ctx->error($builder->errstr);
        $result .= $partial;
    }

    $result;
}

sub hdlr_SheetAssetWorksheets {
    my ( $ctx, $args, $cond ) = @_;
    my $wb = require_current_workbook(@_) or return;
    my $sheets = $wb->worksheets;

    general_loop($sheets, 'sheetasset_worksheet', @_);
}

sub hdlr_SheetAssetRows {
    my ( $ctx, $args, $cond ) = @_;
    my $ws = require_current_worksheet(@_) or return;
    my $rows = $ws->rows || [];

    general_loop($rows, 'sheetasset_row', @_);
}

sub hdlr_SheetAssetColumns {
    my ( $ctx, $args ) = @_;
    my $row = require_current_row(@_) or return;

    general_loop($row, 'sheetasset_cell', @_);
}

sub hdlr_SheetAssetCell {
    my $cell = require_current_cell(@_);
    return unless defined $cell;
    $cell;
}

sub _filter_hashes {
    my ( $ctx, $args, $hashes ) = @_;

    my $field = $args->{filter_field};
    my $value = $args->{filter_value};
    my $regex = $args->{filter_regex};
    my $invert = $args->{filter_invert};
    my $eval = $args->{filter_eval};

    return $hashes unless grep { defined $_ } ($field, $eval);

    my @filtered = @$hashes;
    my ( $true, $false ) = $invert ? ( 0, 1 ) : ( 1, 0 );

    if ( $field && defined $value ) {
        my $is_numeric = $value =~ /^[0-9]+(\.[0-9]+)?$/;
        @filtered = grep {
            ( $is_numeric ? ($_->{$field} == $value) : ($_->{$field} eq $value) ) ? $true : $false;
        } @filtered;
    }

    if ( $field && defined $regex ) {
        @filtered = grep {
            $_->{$field} =~ /$regex/i ? $true : $false;
        } @filtered;
    }

    if ( defined $eval ) {
        my $code = eval "sub { \$_ = shift; $eval; }";
        @filtered = grep {
            $code->($_) ? $true : $false;
        } @filtered if ref $code eq 'CODE';
    }

    \@filtered;
}

sub _sort_hashes {
    my ( $ctx, $args, $hashes ) = @_;

    my $field = $args->{sort_field};
    my $eval = $args->{sort_eval};

    return $hashes unless grep { defined $_ } ( $field, $eval );

    my @sorted = @$hashes;

    if ( $field ) {
        my $order = $args->{sort_order} || 'asc';
        my $as = $args->{sort_as} || 'string';
        my $sorter =
              $order eq 'desc' && $as eq 'number' ? sub { $_[1] <=> $_[0] }
            : $order eq 'desc' && $as ne 'number' ? sub { $_[1] cmp $_[0] }
            : $order ne 'desc' && $as eq 'number' ? sub { $_[0] <=> $_[1] }
            : $order ne 'desc' && $as ne 'number' ? sub { $_[0] cmp $_[1] }
            : undef;

        @sorted = sort { $sorter->($a->{$field}, $b->{$field}) } @sorted;
    }

    if ( defined $eval ) {
        my $sorter = eval "sub { my ( \$a, \$b ) = \@_; $eval; }";
        @sorted = sort {
            $sorter->($a, $b);
        } @sorted if ref $sorter eq 'CODE';
    }

    \@sorted;
}

sub hdlr_SheetAssetTable {
    my ( $ctx, $args, $cond ) = @_;
    my $wb = require_current_workbook(@_) or return;
    my $ws = $wb->current_worksheet or return;

    my $hashes = $args->{workbook} ? $wb->hashes($args) : $ws->hashes($args);
    $hashes = _filter_hashes($ctx, $args, $hashes);
    $hashes = _sort_hashes($ctx, $args, $hashes);

    general_loop($hashes, 'sheetasset_record', @_);
}

sub hdlr_SheetAssetField {
    my ( $ctx, $args ) = @_;

    my $record = require_current_record(@_) or return;
    defined( my $name = $args->{name} )
        or return $ctx->error(plugin->translate('mt:[_1] requires [_2] modifier.', $ctx->stash('tag'), 'name'));

    $record->{$name};
}

sub hdlr_loop_var_condition {
    my ( $var, $ctx, $args ) = @_;
    my $vars = $ctx->{__stash}{vars} || {};
    defined ( my $value = $vars->{$var} )
        or return $ctx->error(plugin->translate('Mo SheetAsset loop context.'));
    $value ? 1: 0;
}

sub hdlr_SheetAssetHeader {
    hdlr_loop_var_condition('__first__', @_);
}

sub hdlr_SheetAssetFooter {
    hdlr_loop_var_condition('__last__', @_);
}

1;