package MT::SheetAsset::Type::Excel;

use strict;
use warnings;
use Encode;

use MT;
use MT::SheetAsset::Model::Workbook;
use MT::SheetAsset::Model::Worksheet;

use base qw( MT::SheetAsset::Type::Base MT::Asset );

use Spreadsheet::Read;
use MT::SheetAsset::Util;

__PACKAGE__->install_properties(
    { class_type => 'sheetasset.xls', child_of => [ 'MT::Blog', 'MT::Website' ], } );

our %EXCEL_TYPES = (
    xls => {
        modules => [ 'Spreadsheet::ParseExcel' ],
        workbook_handler => 'sheetasset_workbook_xls',
    },
    xlsx => {
        modules => [ 'Spreadsheet::ParseXLSX' ],
        workbook_handler => 'sheetasset_workbook_xlsx',
    },
);

sub extensions {
    my $pkg = shift;
    my @extensions = $pkg->sheetasset_available_extensions;
    return $pkg->SUPER::extensions(
        [ map { qr/$_/i } @extensions ]
    );
}

sub class_label {
    my $pkg = shift;
    my @extensions = $pkg->sheetasset_available_extensions;
    plugin->translate('[_1] ([_2])',
        plugin->translate('Excel File'),
        $pkg->sheetasset_join_extensions,
    );
}

sub class_label_plural {
    my $pkg = shift;
    my @extensions = $pkg->sheetasset_available_extensions;
    plugin->translate('[_1] ([_2])',
        plugin->translate('Excel Files'),
        $pkg->sheetasset_join_extensions,
    );
}

sub sheetasset_available_extensions {
    my $pkg = shift;

    my @extensions = grep {
        my $type = $EXCEL_TYPES{$_};
        $pkg->sheetasset_modules_available($type->{modules});
    } sort keys %EXCEL_TYPES;

    @extensions;
}

sub sheetasset_join_extensions {
    my $pkg = shift;
    my @extensions = $pkg->sheetasset_available_extensions;
    @extensions
        ? join( ', ', map { ".$_" } @extensions )
        : plugin->translate('Modules Error');
}

sub sheetasset_modules_available {
    my $self_or_pkg = shift;
    my $modules = shift || [];
    foreach my $m ( @$modules ) {
        eval("require $m; 1;") || return 0;
    }
    1;
}

sub sheetasset_available {
    my $self = shift;
    my $modules = $self->sheetasset_required_modules or return;
    $self->sheetasset_modules_available($modules);
}

sub sheetasset_excel_type {
    my $self = shift;
    my $lc_ext = lc($self->file_ext);
    $EXCEL_TYPES{$lc_ext}
        || $self->error(plugin->translate('Undefined Excel file type whose extension "[_1]".', $lc_ext));
}

sub sheetasset_required_modules {
    my $self = shift;
    my $type = $self->sheetasset_excel_type() or return;
    $type->{modules} || [];
}

sub sheetasset_workbook_handler {
    my $self = shift;
    my $type = $self->sheetasset_excel_type(@_) or return;
    $type->{workbook_handler};
}

sub sheetasset_workbook {
    my $self = shift;
    my $opts = shift || {};

    if ( $self->sheetasset_available ) {
        my $handler = $self->sheetasset_workbook_handler
            or return $self->error(plugin->translate('Cannot find workbook handler.'));
        return $self->$handler($opts);
    } else {
        my $modules = $self->sheetasset_required_modules;
        return $self->error(
            plugin->translate(
                'Excel file type "[_1]" requires modules: [_2].',
                $self->file_ext, join(',', @$modules)
            )
        );
    }
}

sub read_excel_file {
    my $self = shift;
    my $opts = shift || {};
    my $path = $self->readable_path or return;

    my $wb = MT::SheetAsset::Model::Workbook->new({ filepath => $path });
    my $book = ReadData($path);
    return $self->error($book->[0]{error}[1]) if $book->[0]{error};

    for ( my $i = 1; $i <= $book->[0]{sheets}; $i++ ) {
        my $sheet = $book->[$i] or next;
        my @rows = Spreadsheet::Read::rows($sheet);

        $opts->{decoder}->(\@rows) if $opts->{decoder};

        my $ws = MT::SheetAsset::Model::Worksheet->new({ name => $sheet->{label} || '', rows => \@rows });
        $wb->add_worksheet($ws);
    }

    $wb;
}

sub sheetasset_workbook_xls {
    my $self = shift;
    my $opts = shift || {};

    require Encode::Guess;
    require Spreadsheet::ParseExcel;

    $opts->{decoder} = sub {
        my ( $rows ) = @_;
        foreach my $row ( @$rows ) {
            my $cols = scalar @$row;
            for ( my $j = 0; $j < $cols; $j++ ) {
                my $enc = Encode::Guess::guess_encoding($row->[$j]);
                if (ref $enc && $enc->name ne 'ascii') {
                    if ($enc->name eq 'utf8') {
                        $row->[$j] =  Encode::is_utf8($row->[$j]) ? $row->[$j] : decode_utf8($row->[$j]);
                    } else {
                        $row->[$j] = Encode::decode($enc->name, $row->[$j]);
                    }
                }
            }
        }
    };

    $self->read_excel_file($opts, @_);
}

sub _parse_shared_strings_020_minus {
        # Spreadsheet::ParseXLSX 0.20までの実装
        my $self = shift;
        my ($strings) = @_;

        my $PkgStr = [];

        if ($strings) {
            my $xml = XML::Twig->new(
                twig_handlers => {
                    'si' => sub {
                        my ( $twig, $si ) = @_;

                        # XXX this discards information about formatting within cells
                        # not sure how to represent that
                        push @$PkgStr,
                        join( '', map { $_->text } $si->find_nodes('./t') ); # 元は .//t だがこれだとルビも含まれる
                        $twig->purge;
                    },
                }
            );
            $xml->parse( $strings );
        }
        return $PkgStr;
    };

sub _get_text_and_rich_font_by_cell_021_plus {
    my $self = shift;
    my ($si, $theme_colors) = @_;

    # XXX
    my %default_font_opts = (
        Height         => 12,
        Color          => '#000000',
        Name           => '',
        Bold           => 0,
        Italic         => 0,
        Underline      => 0,
        UnderlineStyle => 0,
        Strikeout      => 0,
        Super          => 0,
    );

    my $string_text = '';
    my @rich_font_by_cell;
    my @nodes_r = $si->find_nodes('./s:r');
    if (@nodes_r > 0) {
        for my $chunk (map { $_->children } @nodes_r) {
            my $string_length = length($string_text);
            if ($chunk->name eq 's:t') {
                if (!@rich_font_by_cell) {
                    push @rich_font_by_cell, [
                        $string_length,
                        Spreadsheet::ParseExcel::Font->new(%default_font_opts)
                    ];
                }
                $string_text .= $chunk->text;
            }
            elsif ($chunk->name eq 's:rPr') {
                my %format_text = %default_font_opts;
                for my $node_format ($chunk->children) {
                    if ($node_format->name eq 's:sz') {
                        $format_text{Height} = $node_format->att('val');
                    }
                    elsif ($node_format->name eq 's:color') {
                        $format_text{Color} = $self->_color(
                            $theme_colors,
                            $node_format
                        );
                    }
                    elsif ($node_format->name eq 's:rFont') {
                        $format_text{Name} = $node_format->att('val');
                    }
                    elsif ($node_format->name eq 's:b') {
                        $format_text{Bold} = 1;
                    }
                    elsif ($node_format->name eq 's:i') {
                        $format_text{Italic} = 1;
                    }
                    elsif ($node_format->name eq 's:u') {
                        $format_text{Underline} = 1;
                        if (defined $node_format->att('val')) {
                            $format_text{UnderlineStyle} = 2;
                        } else {
                            $format_text{UnderlineStyle} = 1;
                        }
                    }
                    elsif ($node_format->name eq 's:strike') {
                        $format_text{Strikeout} = 1;
                    }
                    elsif ($node_format->name eq 's:vertAlign') {
                        if ($node_format->att('val') eq 'superscript') {
                            $format_text{Super} = 1;
                        }
                        elsif ($node_format->att('val') eq 'subscript') {
                            $format_text{Super} = 2;
                        }
                    }
                }
                push @rich_font_by_cell, [
                    $string_length,
                    Spreadsheet::ParseExcel::Font->new(%format_text)
                ];
            }
        }
    }
    else {
        $string_text = join '', map { $_->text } $si->find_nodes('./s:t');
    }

    return (
        String => $string_text,
        Rich => \@rich_font_by_cell,
    );
}

sub sheetasset_workbook_xlsx {
    my $self = shift;

    require Spreadsheet::ParseXLSX;

    no warnings 'redefine';
    my $version = $Spreadsheet::ParseXLSX::VERSION;
    my %original = (
        _parse_shared_strings => \&Spreadsheet::ParseXLSX::_parse_shared_strings,
        _get_text_and_rich_font_by_cell => \&Spreadsheet::ParseXLSX::_get_text_and_rich_font_by_cell,
    );

    local *Spreadsheet::ParseXLSX::_parse_shared_strings = $version <= 0.20 ? \&_parse_shared_strings_020_minus : $original{_parse_shared_strings};
    local *Spreadsheet::ParseXLSX::_get_text_and_rich_font_by_cell = $version >= 0.21 ? \&_get_text_and_rich_font_by_cell_021_plus : $original{_get_text_and_rich_font_by_cell};
    $self->read_excel_file(@_);
}


1;