package MT::SheetAsset::URLHandler::Base;

use strict;
use warnings;

use File::Spec;

use MT;
use MT::Asset;
use MT::Util;
use MT::FileMgr;

use MT::SheetAsset::Util;

use base qw( MT::ErrorHandler );

sub new {
    my $package = shift;
    my $self = $package->SUPER::new(@_);

    my ( $opts ) = @_;
    $opts ||= {};
    $self->{$_} = $opts->{$_} foreach keys %$opts;

    $self;
}

sub url_handlers {
    my $package = shift;

    my @handlers;

    my $reg = MT->instance->registry('sheetasset', 'url_handlers') || {};
    my @keys = sort { ( $reg->{$a}->{order} ||= 1000 ) <=> ( $reg->{$b}->{order} || 1000 ) } keys %$reg;

    foreach my $key ( @keys ) {
        my $config = $reg->{$key};
        $config->{key} = $key;

        my $pkg = $config->{package} || next;
        eval "require $pkg; 1" or next;
        my $handler = $pkg->new($config);

        push @handlers, $handler;
    }

    \@handlers;
}

sub labeling {
    my $self = shift;
    my $name = shift;
    my $l = $self->{$name};
    return $l->() if ref $l eq 'CODE';

    my $plugin = $self->{plugin} || plugin;
    return $plugin->translate($l) if $l;

    my @pkg = split(/::/, ref $self);
    $plugin->translate(pop @pkg);
}

sub label { shift->labeling('label') || plugin->translate('Unknown'); }
sub instruction { shift->labeling('instruction') || ''; }
sub example { shift->{example} || ''; }

sub download {
    my $self = shift;
    my ( $ctx ) = @_;

    $ctx->{original_url} = $ctx->{url};
    $self->pre_download($ctx) or return;
    $self->http_request($ctx) or return;
    $self->post_download($ctx) or return;
    $self->save($ctx) or return;

    $ctx->{filepath};
}

sub download_asset {
    my $self = shift;
    my ( $ctx ) = @_;

    # Look up cache
    my $request_cache = MT->instance->request->cache;
    my $cache = $request_cache->{sheetasset_assets_by_url} ||= {};
    my $url = $ctx->{original_url} || $ctx->{url};
    return $cache->{$url} if $cache->{$url};

    $self->download(@_) or return;
    my $asset_pkg = MT::Asset->handler_for_file($ctx->{filename})
        or return $self->error( plugin->translate('Unknown file type.') );

    my $asset = $asset_pkg->new;
    return $self->error( plugin->translate('URL [_1] not points SheetAsset file.', $ctx->{original_url}) )
        unless $asset->can('sheetasset_workbook');

    $asset->file_path($ctx->{filepath});
    $asset->file_name($ctx->{filename});
    $asset->file_ext($ctx->{ext});
    $asset->label($ctx->{filename});

    # Cache it
    $cache->{$url} = $asset;

    $asset;
}

sub detect_url_handler {
    my $package = shift;
    my ( $ctx ) = @_;

    my $handlers = $package->url_handlers;
    my $handler;
    foreach my $h ( @$handlers ) {
        if ( $h->match($ctx) ) {
            $handler = $ctx->{url_handler} = $h;
            last;
        }
    }

    $handler;
}

sub match {
    my $self = shift;
    my ( $ctx ) = @_;

    0;
}

sub pre_download {
    my $self = shift;
    my ( $ctx ) = @_;

    1;
}

sub post_download {
    my $self = shift;
    my ( $ctx ) = @_;

    1;
}

sub temp_dir {
    my $self = shift;
    my $temp_dir = MT->instance->config('TempDir');
    return $self->error(plugin->translate('"[_1]" is not wraitable directory.')) if !-d $temp_dir || !-w $temp_dir;

    my $base = 'sheetasset';
    my $base_dir = File::Spec->catdir($temp_dir, $base);
    mkdir($base_dir) if !-e $base_dir;
    return $self->error(plugin->translate('"[_1]" is not wraitable directory.')) if !-d $base_dir || !-w $base_dir;

    $base_dir;
}

sub temp_filepath_for_url {
    my $self = shift;
    my ( $ctx ) = @_;

    my $digest = Digest::MD5::md5_hex($ctx->{original_url})
        || return $self->error(plugin->translate('No origianl URL.')); # TODO translate
    my $ext = $ctx->{ext} || 'unknown';
    my $filename = "$digest.$ext";

    my $temp_dir = $ctx->{temp_dir} || $self->temp_dir || return;
    my $filepath = $ctx->{filepath} = File::Spec->catdir($temp_dir, $filename);

    $filepath;
}

sub http_request {
    my $self = shift;
    my ( $ctx ) = @_;

    my $ua_config = $ctx->{ua_config} || {};
    $ua_config->{max_size} ||= $ctx->{http_max_size} || MT->instance->config('SheetAssetDownloadMaxSize');
    my $ua = MT->new_ua($ua_config);

    my $url = $ctx->{url} || return $self->error(plugin->translate('No URL')); # TODO translate
    my $req = $ctx->{http_request} = HTTP::Request->new( GET => $url );
    my $res = $ctx->{http_response} = $ua->request($req);

    my $filename = $ctx->{filename} ||= $res->filename;

    if ( $res->is_success ) {
        $ctx->{content} = $res->content;
    } else {
        return $self->error(plugin->translate('Frilure to download url "[_1]".', $url));
    }

    1;
}

sub save {
    my $self = shift;
    my ( $ctx ) = @_;

    my $filename = $ctx->{filename} || '';
    if ( $filename =~ /\.(\w+)$/ ) {
        $ctx->{ext} ||= $1 if $1;
    }

    my $filepath = $ctx->{filepath} ||= $self->temp_filepath_for_url($ctx);
    my $fmgr = MT::FileMgr->new('Local') or return $self->error(pluign->translate('No FileMgr')); # TODO translate
    $fmgr->put_data($ctx->{content}, $filepath)
        || return $self->error(plugin->translate('Failure to save [_1] bytes content to "[_2]".', length($ctx->{content}), $filepath));

    1;
}

1;